@@ -26,6 +26,7 @@
 
 #include "gnome-settings-profile.h"
 #include "gsd-housekeeping-manager.h"
+#include "gsd-disk-space.h"
 
 
 /* General */
@@ -302,6 +303,8 @@ gsd_housekeeping_manager_start (GsdHousekeepingManager *manager,
         g_debug ("Starting housekeeping manager");
         gnome_settings_profile_start (NULL);
 
+        gsd_ldsm_setup (FALSE);
+
         manager->priv->gconf_notify = register_config_callback (manager,
                                       GCONF_THUMB_BINDING_DIR,
                                       (GConfClientNotifyFunc) bindings_callback);
@@ -352,6 +355,8 @@ gsd_housekeeping_manager_stop (GsdHousekeepingManager *manager)
                         do_cleanup (manager);
                 }
         }
+
+        gsd_ldsm_clean ();
 }
 
 
